#!/bin/bash

PRISM_CMD=prism

usage()
{
cat <<-USAGE
usage: run.sh [OPTIONS] ENGINE [PRISM-ARGS]

ENGINE is one of
    sparse
    mtbdd
    hybrid
    explicit

options:
    -p N    only check property with index/name N
    -m M    limit memory usage to M (in MB)
    -t T    limit time to T (in seconds)
    -j N    maximum number of parallel jobs
    -b      just build, do not check properties
    -h      show this help text
USAGE
}

main()
{
    numjobs=0
    props=properties.props

    local opt
    while getopts ":p:m:t:j:bh" opt; do
        case $opt in
            p)
                prop=$OPTARG
                ;;
            m)
                maxmem=$OPTARG
                ;;
            t)
                maxtime=$OPTARG
                ;;
            j)
                numjobs=$OPTARG
                ;;
            b)
                props=
                ;;
            h)
                usage
                exit
                ;;
            \?)
                echo "Invalid option: -$OPTARG" >&2
                exit 1
                ;;
            :)
                echo "Option -$OPTARG requires an argument" >&2
                exit 1
                ;;
        esac
    done
    shift $((OPTIND-1))

    if [[ $# == 0 ]]; then
        usage
        exit 1
    fi

    engine=$1
    shift

    read_dist_settings
    set_resource_limits

    if [[ -z $props ]]; then
        prop=0
        run
    elif [[ -n $prop ]]; then
        run
    else
        prop_count=$(tr -cd ';' < properties.props | wc -c)
        for prop in $(seq 1 $prop_count); do run; done
    fi

    archive
}

run()
{
    if [[ $prop != 0 ]]; then
        echo "property $prop"
    else
        echo "build"
    fi

    make_logdirs
    all_in_one "$@"
    # one_by_one "$@"

    if [[ -n $props ]]; then
        one_by_one_parallel "$@"
    fi
}

all_in_one()
{
    echo " all_in_one"
    for i in $(seq $from $to); do
        echo "  $i"
        logfile="$logdir/all_in_one/${i}.log"
        if ! with_log 0 "$logfile" $LIMIT $PRISM "$@" all_in_one/${i}.prism $props -property $prop; then
            return
        fi
    done
}

one_by_one()
{
    echo " one_by_one"
    for i in $(seq $from $to); do
        if [[ ! -d one_by_one/$i ]]; then continue; fi
        echo "  $i"
        ls -1v one_by_one/$i/*.prism | while read m; do
            name=$(basename $m)
            name=${name%%.*}
            if ! with_log 0 "$logdir/one_by_one/$i/${name}.log" $LIMIT $PRISM "$@" $m $props -property $prop; then
                exit 1
            fi
        done

        if [[ $? == 1 ]]; then break; fi
    done
}

one_by_one_parallel()
{
    echo " parallel"
    for i in $(seq $from $to); do
        echo "  $i"

        if [[ $noredundant == 1 ]]; then
            dir="one_by_one/$to"
        else
            dir="one_by_one/$i"
        fi

        (
        cd $dir
        if [[ $noredundant == 1 ]]; then
            ls -1v *.prism | head -n $((i - from + 1)) > ../../list
        else
            ls -1v *.prism > ../../list
        fi
        )

        if ! with_log 1 "$logdir/parallel/${i}.log" $LIMIT parallel --halt 2 --results $logdir/one_by_one/$i -a list -j $numjobs time $PRISM "$@" $dir/{} $props -property $prop; then
            break
        fi

        for d in $logdir/one_by_one/$i/1/*.prism; do
            name=$(basename $d)
            name=${name%%.*}
            cat "$d/stdout" "$d/stderr" > "$logdir/one_by_one/$i/${name}.log"
        done
    done
    rm list
}

archive()
{
    cp from to logs
    if [[ -e noredundant ]]; then touch logs/noredundant; fi
    tar -czf logs.tar.gz logs/
}

read_dist_settings()
{
    from=$(<from)
    to=$(<to)
    if [[ -e noredundant ]]; then noredundant=1; fi
}

set_resource_limits()
{
    if [[ -n $maxmem ]]; then
        cuddmaxmem=$((maxmem*1000))
    else
        cuddmaxmem=4096000
    fi

    if [[ -n $maxtime ]]; then
        LIMIT="timeout -k 5 $maxtime"
    fi

    PRISM="$PRISM_CMD -cuddmaxmem $cuddmaxmem -${engine}"
}

make_logdirs()
{
    logdir=logs/$prop/$engine
    mkdir -p "${logdir}/all_in_one" "${logdir}/one_by_one" "${logdir}/parallel"

    for i in $(seq $from $to); do
        if [[ -d one_by_one/$i ]]; then
            mkdir -p $logdir/one_by_one/$i
        fi
    done
}

with_log()
{
    local error_only=$1; shift
    local logfile=$1; shift

    if [[ -e $logfile ]]; then
        logfile=$(basename $logfile)
        echo "   Skipping ${logfile%%.*}"
        return
    fi

    if [[ $error_only == 1 ]]; then
        { time "$@" ; } 2> "$logfile" 1> /dev/null
    else
        { time "$@" ; } &> "$logfile"
    fi
    retval=$?

    case $retval in
        124 | 137) # timeout has killed PRISM
            sleep 6 # wait till PRISM has shut down
            echo "TIMEOUT" >> "$logfile"
            ;;
        134) # crash because of insufficient memory
            echo "MEMOUT" >> "$logfile"
            ;;
        101) # erroneously returned on success by parallel (bug #45619)
            retval=0
            ;;
    esac

    return $retval
}

main "$@"

