#!/bin/bash

usage()
{
cat <<-USAGE
usage: time.sh [OPTIONS] LOGDIR ENGINE

ENGINE is one of
    sparse
    mtbdd
    hybrid
    explicit

options:
    -h      show this help text
USAGE
}

HEADER='# size construction checking sum real'
HEADER_PARALLEL='# size real'

REGEX_CONSTR='construction: .* seconds'
REGEX_CHECK='checking: .* seconds'

main()
{
    local opt
    while getopts ":h" opt; do
        case $opt in
            h)
                usage
                exit
                ;;
            \?)
                echo "Invalid option: -$OPTARG" >&2
                exit 1
                ;;
            :)
                echo "Option -$OPTARG requires an argument" >&2
                exit 1
                ;;
        esac
    done
    shift $((OPTIND-1))

    if [[ $# != 2 ]]; then
        usage
        exit 1
    fi

    logdir=$1
    engine=$2

    read_dist_settings
    mkdir -p results

    for prop in $logdir/*; do
        if [[ ! -d $prop ]]; then continue; fi

        p=$(basename $prop)
        resultdir=results/$p/$engine

        if [[ ! -d $logdir/$p/$engine ]]; then continue; fi

        mkdir -p $resultdir

        echo "property $p"

        all_in_one
        if [[ -z $noredundant ]]; then
            one_by_one
        else
            one_by_one_noredundant
        fi
        parallel
    done
}

all_in_one()
{
    echo " all-in-one"
    local datfile=$resultdir/time_all_in_one.dat
    echo $HEADER > $datfile
    for i in $(seq $from $to); do
        f=$logdir/$p/$engine/all_in_one/${i}.log
        if is_incomplete $f; then break; fi

        local constr=$(grep "$REGEX_CONSTR" $f | cut -f 5 -d' ')
        local check=$(grep "$REGEX_CHECK" $f | cut -f 5 -d' ')
        local sum=$(echo $constr + $check | bc)
        local real=$(grep 'real' $f | cut -f 2 | sed 's/m/*60+/' | sed 's/s//' | bc)

        echo $i $constr $check $sum $real >> $datfile
    done
}

one_by_one()
{
    echo " one-by-one"
    local datfile=$resultdir/time_one_by_one.dat
    echo $HEADER > $datfile
    for i in $(seq $from $to); do
        local dir=$logdir/$p/$engine/one_by_one/$i
        if is_incomplete $dir; then break; fi

        local constr=$(grep "$REGEX_CONSTR" $dir/*.log --no-filename | cut -f 5 -d' ' | paste -s -d+ - | bc)
        local check=$(grep "$REGEX_CHECK" $dir/*.log --no-filename | cut -f 5 -d' ' | paste -s -d+ - | bc)
        local sum=$(echo $constr + $check | bc)
        local real=$(grep 'real' $dir/*.log --no-filename | cut -f 2 | sed 's/m/*60+/' | sed 's/s//' | paste -s -d+ - | bc)
        echo $i $constr $check $sum $real >> $datfile
    done
}

one_by_one_noredundant()
{
    # TODO: laufende Summe verwenden, statt immer neu zu berechnen
    echo " one-by-one"
    local datfile=$resultdir/time_one_by_one.dat
    local dir=$logdir/$p/$engine/one_by_one/$to
    echo $HEADER > $datfile
    for i in $(seq $from $to); do
        local n=$((i - from + 1))
        if is_incomplete $dir/$(ls -1v $dir | head -n $n | tail -1); then break; fi

        ls -1v $dir/*.log | head -n $n > list
        local constr=$(xargs -a list grep "$REGEX_CONSTR" --no-filename | cut -f 5 -d' ' | paste -s -d+ - | bc)
        local check=$(xargs -a list grep "$REGEX_CHECK" --no-filename | cut -f 5 -d' ' | paste -s -d+ - | bc)
        local sum=$(echo $constr + $check | bc)
        local real=$(xargs -a list grep 'real' --no-filename | cut -f 2 | sed 's/m/*60+/' | sed 's/s//' | paste -s -d+ - | bc)
        echo $i $constr $check $sum $real >> $datfile
    done
    rm list
}

parallel()
{
    echo " parallel"
    local datfile=$resultdir/time_parallel.dat
    local dir=$logdir/$p/$engine/parallel
    echo $HEADER_PARALLEL > $datfile
    for i in $(seq $from $to); do
        local f=$dir/${i}.log
        if is_incomplete $f; then break; fi

        real=$(grep 'real' $f | tail -n1 | cut -f 2 | sed 's/m/*60+/' | sed 's/s//' | paste -s -d+ - | bc)
        echo $i $real >> $datfile
    done
}

read_dist_settings()
{
    from=$(< $logdir/from)
    to=$(< $logdir/to)
    if [[ -e $logdir/noredundant ]]; then noredundant=1; fi
}

is_incomplete()
{
    if grep -r TIMEOUT $1 > /dev/null; then
        echo "  TIMEOUT for family size $i"
        return 0
    elif grep -r MEMOUT $1 > /dev/null; then
        echo "  MEMOUT for family size $i"
        return 0
    fi

    return 1
}

main "$@"

